/*
 * Copyright (c) Andrey Kuznetsov. All Rights Reserved.
 *
 * http://uio.imagero.com
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  o Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 *  o Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 *  o Neither the name of Andrey Kuznetsov nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package com.imagero.uio;

import java.io.IOException;
import java.io.InputStream;
import java.io.PrintStream;

/**
 * @author Andrei Kouznetsov
 *         Date: 05.07.2004
 *         Time: 15:02:01
 */
public class Sys {
    public static PrintStreamFilter out = new PrintStreamFilter(java.lang.System.out);
    public static PrintStreamFilter err = new PrintStreamFilter(java.lang.System.err);
    public static InputStream in = java.lang.System.in;

    public static PrintStreamFilter getErr() {
        return err;
    }

    public static void setErr(PrintStream err) {
        Sys.err = new PrintStreamFilter(err);
    }

    public static PrintStreamFilter getOut() {
        return out;
    }

    public static void setOut(PrintStream out) {
        Sys.out = new PrintStreamFilter(out);
    }

    public static InputStream getIn() {
        return in;
    }

    public static void setIn(InputStream in) {
        Sys.in = in;
    }

    public static class PrintStreamFilter {
        PrintStream ps;

        public PrintStreamFilter(PrintStream ps) {
            this.ps = ps;
        }

        public PrintStream getPrintStream() {
            return ps;
        }

        public void print(String x) {
            if (x == null) {
                ps.print(x);
                return;
            }
            char[] chars = new char[x.length()];
            x.getChars(0, x.length(), chars, 0);
            int p = chars.length;
            for (int j = 0; j < p; j++) {
                char c = chars[j];
                if (Character.isLetterOrDigit(c)) {
                    chars[j] = c;
                }
                else if (Character.isWhitespace(c)) {
                    chars[j] = ' ';
                }
                else if (Character.isISOControl(c)) {
                    chars[j] = '.';
                }
                else {
                    chars[j] = c;
                }
            }
            ps.print(chars);
        }

        public void println(String x) {
            print(x);
            ps.println();
        }

        public void println() {
            ps.println();
        }

        public void flush() {
            ps.flush();
        }

        public void close() {
            ps.close();
        }

        public boolean checkError() {
            return ps.checkError();
        }

        public void write(int b) {
            ps.write(b);
        }

        public void write(byte buf[], int off, int len) {
            ps.write(buf, off, len);
        }

        public void print(boolean b) {
            ps.print(b);
        }

        public void print(char c) {
            ps.print(c);
        }

        public void print(int i) {
            ps.print(i);
        }

        public void print(long l) {
            ps.print(l);
        }

        public void print(float f) {
            ps.print(f);
        }

        public void print(double d) {
            ps.print(d);
        }

        public void print(char s[]) {
            ps.print(s);
        }

        public void print(Object obj) {
            ps.print(obj);
        }

        public void println(boolean x) {
            ps.println(x);
        }

        public void println(char x) {
            ps.println(x);
        }

        public void println(int x) {
            ps.println(x);
        }

        public void println(long x) {
            ps.println(x);
        }

        public void println(float x) {
            ps.println(x);
        }

        public void println(double x) {
            ps.println(x);
        }

        public void println(char x[]) {
            ps.println(x);
        }

        public void println(Object x) {
            ps.println(x);
        }

        public void write(byte b[]) throws IOException {
            ps.write(b);
        }
    }
}
